@@ -12,29 +12,37 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import threading
 import ipaddress
+import threading
+
 from datetime import datetime
-from libra.common.json_gearman import JSONGearmanClient
 from gearman.constants import JOB_UNKNOWN
+from oslo.config import cfg
 from sqlalchemy import func
+
 from libra.common.api.lbaas import Device, PoolBuilding, Vip, db_session
+from libra.common.json_gearman import JSONGearmanClient
+from libra.openstack.common import log
 
 #TODO: Lots of duplication of code here, need to cleanup
 
+LOG = log.getLogger(__name__)
+
 
 class Pool(object):
 
-    DELETE_SECONDS = 05
-    PROBE_SECONDS = 30
-    VIPS_SECONDS = 50
+    DELETE_SECONDS = cfg.CONF['admin_api'].delete_timer_seconds
+    PROBE_SECONDS = cfg.CONF['admin_api'].probe_timer_seconds
+    VIPS_SECONDS = cfg.CONF['admin_api'].vips_timer_seconds
 
-    def __init__(self, logger, args):
-        self.logger = logger
-        self.args = args
+    def __init__(self):
         self.probe_timer = None
         self.delete_timer = None
         self.vips_time = None
+        self.server_id = cfg.CONF['admin_api']['server_id']
+        self.number_of_servers = cfg.CONF['admin_api']['number_of_servers']
+        self.vip_pool_size = cfg.CONF['admin_api']['vip_pool_size']
+        self.node_pool_size = cfg.CONF['admin_api']['node_pool_size']
 
         self.start_delete_sched()
         self.start_probe_sched()
@@ -51,11 +59,11 @@ class Pool(object):
     def delete_devices(self):
         """ Searches for all devices in the DELETED state and removes them """
         minute = datetime.now().minute
-        if self.args.server_id != minute % self.args.number_of_servers:
-            self.logger.info('Not our turn to run delete check, sleeping')
+        if self.server_id != minute % self.number_of_servers:
+            LOG.info('Not our turn to run delete check, sleeping')
             self.start_delete_sched()
             return
-        self.logger.info('Running device delete check')
+        LOG.info('Running device delete check')
         try:
             message = []
             with db_session() as session:
@@ -70,71 +78,71 @@ class Pool(object):
                     message.append(dict(task='libra_pool_mgm', data=job_data))
                 session.commit()
             if not message:
-                self.logger.info("No devices to delete")
+                LOG.info("No devices to delete")
             else:
-                gear = GearmanWork(self.args, self.logger)
+                gear = GearmanWork()
                 gear.send_delete_message(message)
         except:
-            self.logger.exception("Exception when deleting devices")
+            LOG.exception("Exception when deleting devices")
 
         self.start_delete_sched()
 
     def probe_vips(self):
         minute = datetime.now().minute
-        if self.args.server_id != minute % self.args.number_of_servers:
-            self.logger.info('Not our turn to run vips check, sleeping')
+        if self.server_id != minute % self.number_of_servers:
+            LOG.info('Not our turn to run vips check, sleeping')
             self.start_vips_sched()
             return
-        self.logger.info('Running vips count probe check')
+        LOG.info('Running vips count probe check')
         try:
             with db_session() as session:
                 NULL = None  # For pep8
                 vip_count = session.query(Vip).\
                     filter(Vip.device == NULL).count()
-                if vip_count >= self.args.vip_pool_size:
-                    self.logger.info("Enough vips exist, no work to do")
+                if vip_count >= self.vip_pool_size:
+                    LOG.info("Enough vips exist, no work to do")
                     session.commit()
                     self.start_vips_sched()
                     return
 
-                build_count = self.args.vip_pool_size - vip_count
+                build_count = self.vip_pool_size - vip_count
                 self._build_vips(build_count)
         except:
-            self.logger.exception(
+            LOG.exception(
                 "Uncaught exception during vip pool expansion"
             )
         self.start_vips_sched()
 
     def probe_devices(self):
         minute = datetime.now().minute
-        if self.args.server_id != minute % self.args.number_of_servers:
-            self.logger.info('Not our turn to run probe check, sleeping')
+        if self.server_id != minute % self.number_of_servers:
+            LOG.info('Not our turn to run probe check, sleeping')
             self.start_probe_sched()
             return
-        self.logger.info('Running device count probe check')
+        LOG.info('Running device count probe check')
         try:
             with db_session() as session:
                 # Double check we have no outstanding builds assigned to us
                 session.query(PoolBuilding).\
-                    filter(PoolBuilding.server_id == self.args.server_id).\
+                    filter(PoolBuilding.server_id == self.server_id).\
                     delete()
                 session.flush()
                 dev_count = session.query(Device).\
                     filter(Device.status == 'OFFLINE').count()
-                if dev_count >= self.args.node_pool_size:
-                    self.logger.info("Enough devices exist, no work to do")
+                if dev_count >= self.node_pool_size:
+                    LOG.info("Enough devices exist, no work to do")
                     session.commit()
                     self.start_probe_sched()
                     return
 
-                build_count = self.args.node_pool_size - dev_count
+                build_count = self.node_pool_size - dev_count
                 built = session.query(func.sum(PoolBuilding.qty)).first()
                 if not built[0]:
                     built = 0
                 else:
                     built = built[0]
                 if build_count - built <= 0:
-                    self.logger.info(
+                    LOG.info(
                         "Other servers are building enough nodes"
                     )
                     session.commit()
@@ -142,7 +150,7 @@ class Pool(object):
                     return
                 build_count -= built
                 building = PoolBuilding()
-                building.server_id = self.args.server_id
+                building.server_id = self.server_id
                 building.qty = build_count
                 session.add(building)
                 session.commit()
@@ -152,11 +160,11 @@ class Pool(object):
             self._build_nodes(build_count)
             with db_session() as session:
                 session.query(PoolBuilding).\
-                    filter(PoolBuilding.server_id == self.args.server_id).\
+                    filter(PoolBuilding.server_id == self.server_id).\
                     delete()
                 session.commit()
         except:
-            self.logger.exception("Uncaught exception during pool expansion")
+            LOG.exception("Uncaught exception during pool expansion")
         self.start_probe_sched()
 
     def _build_nodes(self, count):
@@ -166,7 +174,7 @@ class Pool(object):
         while it < count:
             message.append(dict(task='libra_pool_mgm', data=job_data))
             it += 1
-        gear = GearmanWork(self.args, self.logger)
+        gear = GearmanWork()
         gear.send_create_message(message)
 
     def _build_vips(self, count):
@@ -176,7 +184,7 @@ class Pool(object):
         while it < count:
             message.append(dict(task='libra_pool_mgm', data=job_data))
             it += 1
-        gear = GearmanWork(self.args, self.logger)
+        gear = GearmanWork()
         gear.send_vips_message(message)
 
     def start_probe_sched(self):
@@ -186,8 +194,7 @@ class Pool(object):
         else:
             sleeptime = 60 - (seconds - self.PROBE_SECONDS)
 
-        self.logger.info('Pool probe check timer sleeping for {secs} seconds'
-                         .format(secs=sleeptime))
+        LOG.info('Pool probe check timer sleeping for %d seconds', sleeptime)
         self.probe_timer = threading.Timer(sleeptime, self.probe_devices, ())
         self.probe_timer.start()
 
@@ -198,8 +205,7 @@ class Pool(object):
         else:
             sleeptime = 60 - (seconds - self.VIPS_SECONDS)
 
-        self.logger.info('Pool vips check timer sleeping for {secs} seconds'
-                         .format(secs=sleeptime))
+        LOG.info('Pool vips check timer sleeping for %d seconds', sleeptime)
         self.vips_timer = threading.Timer(sleeptime, self.probe_vips, ())
         self.vips_timer.start()
 
@@ -210,33 +216,31 @@ class Pool(object):
         else:
             sleeptime = 60 - (seconds - self.DELETE_SECONDS)
 
-        self.logger.info('Pool delete check timer sleeping for {secs} seconds'
-                         .format(secs=sleeptime))
+        LOG.info('Pool delete check timer sleeping for %d seconds', sleeptime)
         self.delete_timer = threading.Timer(sleeptime, self.delete_devices, ())
         self.delete_timer.start()
 
 
 class GearmanWork(object):
 
-    def __init__(self, args, logger):
-        self.logger = logger
-        if all([args.gearman_ssl_key, args.gearman_ssl_cert,
-                args.gearman_ssl_ca]):
-            # Use SSL connections to each Gearman job server.
-            ssl_server_list = []
-            for server in args.gearman:
-                ghost, gport = server.split(':')
-                ssl_server_list.append({'host': ghost,
-                                        'port': int(gport),
-                                        'keyfile': args.gearman_ssl_key,
-                                        'certfile': args.gearman_ssl_cert,
-                                        'ca_certs': args.gearman_ssl_ca})
-            self.gearman_client = JSONGearmanClient(ssl_server_list)
-        else:
-            self.gearman_client = JSONGearmanClient(args.gearman)
+    def __init__(self):
+        server_list = []
+        for server in cfg.CONF['gearman']['servers']:
+            host, port = server.split(':')
+            server_list.append({'host': host,
+                                'port': int(port),
+                                'keyfile': cfg.CONF['gearman']['ssl_key'],
+                                'certfile': cfg.CONF['gearman']['ssl_cert'],
+                                'ca_certs': cfg.CONF['gearman']['ssl_ca'],
+                                'keepalive': cfg.CONF['gearman']['keepalive'],
+                                'keepcnt': cfg.CONF['gearman']['keepcnt'],
+                                'keepidle': cfg.CONF['gearman']['keepidle'],
+                                'keepintvl': cfg.CONF['gearman']['keepintvl']
+                                })
+        self.gearman_client = JSONGearmanClient(server_list)
 
     def send_delete_message(self, message):
-        self.logger.info("Sending {0} gearman messages".format(len(message)))
+        LOG.info("Sending %d gearman messages", len(message))
         job_status = self.gearman_client.submit_multiple_jobs(
             message, background=False, wait_until_complete=True,
             max_retries=10, poll_timeout=30.0
@@ -244,13 +248,13 @@ class GearmanWork(object):
         delete_count = 0
         for status in job_status:
             if status.state == JOB_UNKNOWN:
-                self.logger.error('Gearman Job server fail')
+                LOG.error('Gearman Job server fail')
                 continue
             if status.timed_out:
-                self.logger.error('Gearman timeout whilst deleting device')
+                LOG.error('Gearman timeout whilst deleting device')
                 continue
             if status.result['response'] == 'FAIL':
-                self.logger.error(
+                LOG.error(
                     'Pool manager failed to delete a device, removing from DB'
                 )
 
@@ -260,13 +264,11 @@ class GearmanWork(object):
                     filter(Device.name == status.result['name']).delete()
                 session.commit()
 
-        self.logger.info(
-            '{nodes} freed devices delete from pool'.format(nodes=delete_count)
-        )
+        LOG.info('%d freed devices delete from pool', delete_count)
 
     def send_vips_message(self, message):
         # TODO: make this gearman part more async, not wait for all builds
-        self.logger.info("Sending {0} gearman messages".format(len(message)))
+        LOG.info("Sending %d gearman messages", len(message))
         job_status = self.gearman_client.submit_multiple_jobs(
             message, background=False, wait_until_complete=True,
             max_retries=10, poll_timeout=3600.0
@@ -274,30 +276,30 @@ class GearmanWork(object):
         built_count = 0
         for status in job_status:
             if status.state == JOB_UNKNOWN:
-                self.logger.error('Gearman Job server fail')
+                LOG.error('Gearman Job server fail')
                 continue
             if status.timed_out:
-                self.logger.error('Gearman timeout whilst building vip')
+                LOG.error('Gearman timeout whilst building vip')
                 continue
             if status.result['response'] == 'FAIL':
-                self.logger.error('Pool manager failed to build a vip')
+                LOG.error('Pool manager failed to build a vip')
                 continue
 
             built_count += 1
             try:
                 self._add_vip(status.result)
             except:
-                self.logger.exception(
+                LOG.exception(
                     'Could not add vip to DB, node data: {0}'
                     .format(status.result)
                 )
-        self.logger.info(
+        LOG.info(
             '{vips} vips built and added to pool'.format(vips=built_count)
         )
 
     def send_create_message(self, message):
         # TODO: make this gearman part more async, not wait for all builds
-        self.logger.info("Sending {0} gearman messages".format(len(message)))
+        LOG.info("Sending {0} gearman messages".format(len(message)))
         job_status = self.gearman_client.submit_multiple_jobs(
             message, background=False, wait_until_complete=True,
             max_retries=10, poll_timeout=3600.0
@@ -305,29 +307,31 @@ class GearmanWork(object):
         built_count = 0
         for status in job_status:
             if status.state == JOB_UNKNOWN:
-                self.logger.error('Gearman Job server fail')
+                LOG.error('Gearman Job server fail')
                 continue
             if status.timed_out:
-                self.logger.error('Gearman timeout whilst building device')
+                LOG.error('Gearman timeout whilst building device')
                 continue
             if status.result['response'] == 'FAIL':
-                self.logger.error('Pool manager failed to build a device')
+                LOG.error('Pool manager failed to build a device')
+                if 'name' in status.result:
+                    self._add_bad_node(status.result)
                 continue
 
             built_count += 1
             try:
                 self._add_node(status.result)
             except:
-                self.logger.exception(
+                LOG.exception(
                     'Could not add node to DB, node data: {0}'
                     .format(status.result)
                 )
-        self.logger.info(
+        LOG.info(
             '{nodes} devices built and added to pool'.format(nodes=built_count)
         )
 
     def _add_vip(self, data):
-        self.logger.info('Adding vip {0} to DB'.format(data['ip']))
+        LOG.info('Adding vip {0} to DB'.format(data['ip']))
         vip = Vip()
         vip.ip = int(ipaddress.IPv4Address(unicode(data['ip'])))
         with db_session() as session:
@@ -335,7 +339,7 @@ class GearmanWork(object):
             session.commit()
 
     def _add_node(self, data):
-        self.logger.info('Adding device {0} to DB'.format(data['name']))
+        LOG.info('Adding device {0} to DB'.format(data['name']))
         device = Device()
         device.name = data['name']
         device.publicIpAddr = data['addr']
@@ -349,3 +353,21 @@ class GearmanWork(object):
         with db_session() as session:
             session.add(device)
             session.commit()
+
+    def _add_bad_node(self, data):
+        LOG.info(
+            'Adding bad device {0} to DB to be deleted'.format(data['name'])
+        )
+        device = Device()
+        device.name = data['name']
+        device.publicIpAddr = data['addr']
+        # TODO: kill this field, make things use publicIpAddr instead
+        device.floatingIpAddr = data['addr']
+        device.az = data['az']
+        device.type = data['type']
+        device.pingCount = 0
+        device.status = 'DELETED'
+        device.created = None
+        with db_session() as session:
+            session.add(device)
+            session.commit()
