@@ -32,9 +32,12 @@ from libra.common.api.lbaas import HealthMonitor
 from libra.common.exc import ExhaustedError
 from libra.api.model.validators import LBPut, LBPost, LBResp, LBVipResp
 from libra.api.model.validators import LBRespNode
-from libra.common.api.gearman_client import submit_job, submit_vip_job
+from libra.common.api.gearman_client import submit_job
+from libra.common.api.mnb import update_mnb
 from libra.api.acl import get_limited_to_project
 from libra.api.library.exp import OverLimit, IPOutOfRange, NotFound
+from libra.api.library.exp import ImmutableEntity, ImmutableStates
+from libra.api.library.exp import ImmutableStatesNoError
 from libra.api.library.ip_filter import ipfilter
 from pecan import conf
 from wsme import types as wtypes
@@ -102,7 +105,7 @@ class LoadBalancersController(RestController):
                     LoadBalancer.updated, LoadBalancer.errmsg,
                     Vip.id.label('vipid'), Vip.ip
                 ).join(LoadBalancer.devices).\
-                    join(Device.vip).\
+                    outerjoin(Device.vip).\
                     filter(LoadBalancer.tenantid == tenant_id).\
                     filter(LoadBalancer.id == self.lbid).\
                     first()
@@ -115,17 +118,26 @@ class LoadBalancersController(RestController):
                 load_balancers['nodeCount'] = session.query(Node).\
                     filter(Node.lbid == load_balancers['id']).count()
 
-                load_balancers['virtualIps'] = [{
-                    "id": load_balancers['vipid'],
-                    "type": "PUBLIC",
-                    "ipVersion": "IPV4",
-                    "address": str(ipaddress.IPv4Address(
-                        load_balancers['ip']
-                    )),
-                }]
-                del(load_balancers['ip'])
-                del(load_balancers['vipid'])
-
+                if load_balancers['vipid']:
+                    load_balancers['virtualIps'] = [{
+                        "id": load_balancers['vipid'],
+                        "type": "PUBLIC",
+                        "ipVersion": "IPV4",
+                        "address": str(ipaddress.IPv4Address(
+                            load_balancers['ip']
+                        )),
+                    }]
+                    del(load_balancers['ip'])
+                    del(load_balancers['vipid'])
+                else:
+                    # We are still assigning a VIP
+                    load_balancers['virtualIps'] = [{
+                        "id": None,
+                        "type": "ASSIGNING",
+                        "ipVersion": "IPV4",
+                        "address": None
+                    }]
+                    del(load_balancers['vipid'])
                 nodes = session.query(
                     Node.id, Node.address, Node.port, Node.status,
                     Node.enabled, Node.weight
@@ -317,26 +329,22 @@ class LoadBalancersController(RestController):
                         session.query(loadbalancers_devices.c.device)
                     )).\
                     filter(Device.status == "OFFLINE").\
+                    filter(Device.pingCount == 0).\
                     with_lockmode('update').\
                     first()
-                NULL = None  # For pep8
-                vip = session.query(Vip).\
-                    filter(Vip.device == NULL).\
-                    with_lockmode('update').\
-                    first()
-                vip.device = device.id
                 if device is None:
                     session.rollback()
                     raise ExhaustedError('No devices available')
-                if vip is None:
-                    session.rollback()
-                    raise ExhaustedError('No virtual IPs available')
-                vip.device = device.id
-                submit_vip_job(
-                    'ASSIGN', device.name, str(ipaddress.IPv4Address(vip.ip))
-                )
+
+                vip = None
             else:
                 virtual_id = body.virtualIps[0].id
+                # Make sure virtual ID is actually an int
+                try:
+                    virtual_id = int(virtual_id)
+                except:
+                    session.rollback()
+                    raise NotFound('Invalid virtual IP provided')
                 # This is an additional load balancer
                 device = session.query(
                     Device
@@ -344,13 +352,6 @@ class LoadBalancersController(RestController):
                     filter(Vip.id == virtual_id).\
                     first()
 
-                if device.status == 'ERROR':
-                    session.rollback()
-                    raise ClientSideError(
-                        'Cannot add a Load Balancer to a device'
-                        ' in an ERROR state'
-                    )
-
                 old_lb = session.query(
                     LoadBalancer
                 ).join(LoadBalancer.devices).\
@@ -358,6 +359,15 @@ class LoadBalancersController(RestController):
                     filter(LoadBalancer.tenantid == tenant_id).\
                     filter(Vip.id == virtual_id).\
                     first()
+
+                if old_lb.status in ImmutableStates:
+                    session.rollback()
+                    raise ImmutableEntity(
+                        'Existing Load Balancer on VIP in a non-ACTIVE state'
+                        ', current state: {0}'
+                        .format(old_lb.status)
+                    )
+
                 vip = session.query(Vip).\
                     filter(Vip.device == device.id).\
                     first()
@@ -415,8 +425,10 @@ class LoadBalancersController(RestController):
                 session.add(out_node)
 
             # now save the loadbalancer_id to the device and switch its status
-            # to online
-            device.status = "ONLINE"
+            # to build so the monitoring does not trigger early.
+            # The gearman message code will switch to ONLINE once we know
+            # everything is good
+            device.status = "BUILD"
             session.flush()
 
             return_data = LBResp()
@@ -428,10 +440,15 @@ class LoadBalancersController(RestController):
             return_data.status = lb.status
             return_data.created = lb.created
             return_data.updated = lb.updated
-            vip_resp = LBVipResp(
-                address=str(ipaddress.IPv4Address(vip.ip)),
-                id=str(vip.id), type='PUBLIC', ipVersion='IPV4'
-            )
+            if vip:
+                vip_resp = LBVipResp(
+                    address=str(ipaddress.IPv4Address(vip.ip)),
+                    id=str(vip.id), type='PUBLIC', ipVersion='IPV4'
+                )
+            else:
+                vip_resp = LBVipResp(
+                    address=None, id=None, type='ASSIGNING', ipVersion='IPV4'
+                )
             return_data.virtualIps = [vip_resp]
             return_data.nodes = []
             for node in body.nodes:
@@ -452,6 +469,10 @@ class LoadBalancersController(RestController):
             submit_job(
                 'UPDATE', device.name, device.id, lb.id
             )
+
+            #Notify billing of the LB creation
+            update_mnb('lbaas.instance.create', lb.id, tenant_id)
+
             return return_data
 
     @wsme_pecan.wsexpose(None, body=LBPut, status_code=202)
@@ -471,6 +492,14 @@ class LoadBalancersController(RestController):
                 session.rollback()
                 raise NotFound('Load Balancer ID is not valid')
 
+            if lb.status in ImmutableStates:
+                session.rollback()
+                raise ImmutableEntity(
+                    'Cannot modify a Load Balancer in a non-ACTIVE state'
+                    ', current state: {0}'
+                    .format(lb.status)
+                )
+
             if body.name != Unset:
                 namelimit = session.query(Limits.value).\
                     filter(Limits.name == 'maxLoadBalancerNameLength').scalar()
@@ -491,12 +520,6 @@ class LoadBalancersController(RestController):
                 filter(LoadBalancer.id == self.lbid).\
                 first()
 
-            if device.status == 'ERROR':
-                session.rollback()
-                raise ClientSideError(
-                    'Cannot update a Load Balancer in an ERROR state'
-                )
-
             session.commit()
             submit_job(
                 'UPDATE', device.name, device.id, lb.id
@@ -530,6 +553,14 @@ class LoadBalancersController(RestController):
             if lb is None:
                 session.rollback()
                 raise NotFound("Load Balancer ID is not valid")
+            # So we can delete ERROR, but not other Immutable states
+            if lb.status in ImmutableStatesNoError:
+                session.rollback()
+                raise ImmutableEntity(
+                    'Cannot delete a Load Balancer in a non-ACTIVE state'
+                    ', current state: {0}'
+                    .format(lb.status)
+                )
             lb.status = 'PENDING_DELETE'
             device = session.query(
                 Device.id, Device.name
@@ -546,11 +577,15 @@ class LoadBalancersController(RestController):
                     filter(Node.lbid == lb.id).delete()
                 session.query(HealthMonitor).\
                     filter(HealthMonitor.lbid == lb.id).delete()
+                session.commit()
             else:
+                session.commit()
                 submit_job(
                     'DELETE', device.name, device.id, lb.id
                 )
-            session.commit()
+
+            #Notify billing of the LB deletion
+            update_mnb('lbaas.instance.delete', lb.id, tenant_id)
             return None
 
     def usage(self, load_balancer_id):
