@@ -14,16 +14,18 @@
 
 import eventlet
 eventlet.monkey_patch()
-import logging
 import ipaddress
 from libra.common.json_gearman import JSONGearmanClient
 from libra.common.api.lbaas import LoadBalancer, db_session, Device, Node, Vip
 from libra.common.api.lbaas import HealthMonitor
 from libra.common.api.lbaas import loadbalancers_devices
-from sqlalchemy.exc import OperationalError
+from libra.openstack.common import log
 from pecan import conf
 
 
+LOG = log.getLogger(__name__)
+
+
 gearman_workers = [
     'UPDATE',  # Create/Update a Load Balancer.
     'SUSPEND',  # Suspend a Load Balancer.
@@ -31,56 +33,79 @@ gearman_workers = [
     'DELETE',  # Delete a Load Balancer.
     'DISCOVER',  # Return service discovery information.
     'ARCHIVE',  # Archive LB log files.
-    'STATS'  # Get load balancer statistics.
+    'STATS',  # Get load balancer statistics.
+    'PING'    # Ping load balancers
 ]
 
 
 def submit_job(job_type, host, data, lbid):
-    logger = logging.getLogger(__name__)
-    eventlet.spawn_n(client_job, logger, job_type, str(host), data, lbid)
+    eventlet.spawn_n(client_job, job_type, str(host), data, lbid)
 
 
 def submit_vip_job(job_type, device, vip):
-    logger = logging.getLogger(__name__)
     eventlet.spawn_n(
-        client_job, logger, job_type, "libra_pool_mgm", device, vip
+        client_job, job_type, "libra_pool_mgm", device, vip
     )
 
 
-def client_job(logger, job_type, host, data, lbid):
-    for x in xrange(5):
-        try:
-            client = GearmanClientThread(logger, host, lbid)
-            logger.info(
-                "Sending Gearman job {0} to {1} for loadbalancer {2}".format(
-                    job_type, host, lbid
-                )
+def client_job(job_type, host, data, lbid):
+    try:
+        client = GearmanClientThread(host, lbid)
+        LOG.info(
+            "Sending Gearman job {0} to {1} for loadbalancer {2}".format(
+                job_type, host, lbid
             )
-            if job_type == 'UPDATE':
-                client.send_update(data)
-            if job_type == 'DELETE':
-                client.send_delete(data)
-            if job_type == 'ARCHIVE':
-                client.send_archive(data)
-            if job_type == 'ASSIGN':
-                client.send_assign(data)
-            if job_type == 'REMOVE':
-                client.send_remove(data)
-            return
-        except OperationalError:
-            # Auto retry on galera locking error
-            logger.warning(
-                "Galera deadlock in gearman, retry {0}".format(x + 1)
-            )
-        except:
-            logger.exception("Gearman thread unhandled exception")
+        )
+        if job_type == 'UPDATE':
+            client.send_update(data)
+        if job_type == 'DELETE':
+            client.send_delete(data)
+        if job_type == 'ARCHIVE':
+            client.send_archive(data)
+        if job_type == 'ASSIGN':
+            # Try the assign 5 times
+            for x in xrange(0, 5):
+                status = client.send_assign(data)
+                if status:
+                    break
+            with db_session() as session:
+                device = session.query(Device).\
+                    filter(Device.name == data).first()
+                if device is None:
+                    LOG.error(
+                        "Device {0} not found in ASSIGN, this shouldn't happen"
+                        .format(data)
+                    )
+                    return
 
-    logger.error("Gearman thread could not talk to DB")
+                if not status:
+                    LOG.error(
+                        "Giving up vip assign for device {0}".format(data)
+                    )
+                    errmsg = 'Floating IP assign failed'
+                    client._set_error(device.id, errmsg, session)
+                else:
+                    lbs = session.query(
+                        LoadBalancer
+                    ).join(LoadBalancer.nodes).\
+                        join(LoadBalancer.devices).\
+                        filter(Device.id == device.id).\
+                        filter(LoadBalancer.status != 'DELETED').\
+                        all()
+                    for lb in lbs:
+                        lb.status = 'ACTIVE'
+                    device.status = 'ONLINE'
+                session.commit()
+
+        if job_type == 'REMOVE':
+            client.send_remove(data)
+        return
+    except:
+        LOG.exception("Gearman thread unhandled exception")
 
 
 class GearmanClientThread(object):
-    def __init__(self, logger, host, lbid):
-        self.logger = logger
+    def __init__(self, host, lbid):
         self.host = host
         self.lbid = lbid
 
@@ -99,29 +124,104 @@ class GearmanClientThread(object):
         self.gearman_client = JSONGearmanClient(server_list)
 
     def send_assign(self, data):
+        NULL = None  # For pep8
+        with db_session() as session:
+            device = session.query(Device).\
+                filter(Device.name == data).first()
+            if device is None:
+                self.LOG.error(
+                    "VIP assign have been given non existent device {0}"
+                    .format(data)
+                )
+                session.rollback()
+                return False
+            if not self.lbid:
+                vip = session.query(Vip).\
+                    filter(Vip.device == NULL).\
+                    with_lockmode('update').\
+                    first()
+                if vip is None:
+                    errmsg = 'Floating IP assign failed (none available)'
+                    LOG.error(
+                        "Failed to assign IP to device {0} (none available)"
+                        .format(data)
+                    )
+                    self._set_error(device.id, errmsg, session)
+                    session.commit()
+                    return False
+            else:
+                vip = session.query(Vip).\
+                    filter(Vip.id == self.lbid).first()
+                if vip is None:
+                    errmsg = 'Cannot find existing floating IP'
+                    self.LOG.error(
+                        "Failed to assign IP to device {0}"
+                        .format(data)
+                    )
+                    self._set_error(device.id, errmsg, session)
+                    session.commit()
+                    return False
+            vip.device = device.id
+            vip_id = vip.id
+            vip_ip = vip.ip
+            session.commit()
+        ip_str = str(ipaddress.IPv4Address(vip_ip))
+
         job_data = {
             'action': 'ASSIGN_IP',
             'name': data,
-            'ip': self.lbid
+            'ip': ip_str
         }
         status, response = self._send_message(job_data, 'response')
-        if not status:
-            self.logger.error(
-                "Failed to assign IP {0} to device {1}".format(self.lbid, data)
+        if status:
+            return True
+        elif self.lbid:
+            self.LOG.error(
+                "Failed to assign IP {0} to device {1}"
+                .format(ip_str, data)
             )
+        else:
+            self.LOG.error(
+                "Failed to assign IP {0} to device {1}"
+                .format(ip_str, data)
+            )
+            # set to device 0 to make sure it won't be used again
+            with db_session() as session:
+                vip = session.query(Vip).filter(Vip.id == vip_id).first()
+                vip.device = 0
+                session.commit()
+            submit_vip_job('REMOVE', None, ip_str)
+        return False
 
-    def send_remove(self, data):
+    def send_remove(self, data=None):
         job_data = {
-            'action': 'REMOVE_IP',
-            'name': data,
+            'action': 'DELETE_IP',
             'ip': self.lbid
         }
-        status, response = self._send_message(job_data, 'response')
-        if not status:
-            self.logger.error(
-                "Failed to remove IP {0} from device {1}"
-                .format(self.lbid, data)
+        ip_int = int(ipaddress.IPv4Address(unicode(self.lbid)))
+        for x in xrange(0, 5):
+            self.logger.info(
+                'Attempt to delete IP {0} #{1}'
+                .format(self.lbid, x)
             )
+            status, response = self._send_message(job_data, 'response')
+            if status:
+                break
+        with db_session() as session:
+            if not status:
+                LOG.error(
+                    "Failed to delete IP {0}"
+                    .format(self.lbid)
+                )
+                # Set to 0 to mark as something that needs cleaning up
+                # but cannot be used again
+                vip = session.query(Vip).\
+                    filter(Vip.ip == ip_int).first()
+                vip.device = 0
+            else:
+                session.query(Vip).\
+                    filter(Vip.ip == ip_int).delete()
+            session.commit()
 
     def send_delete(self, data):
         with db_session() as session:
@@ -129,6 +229,7 @@ class GearmanClientThread(object):
                 LoadBalancer
             ).join(LoadBalancer.devices).\
                 filter(Device.id == data).\
+                filter(LoadBalancer.id != self.lbid).\
                 filter(LoadBalancer.status != 'DELETED').\
                 filter(LoadBalancer.status != 'PENDING_DELETE').\
                 count()
@@ -169,10 +270,10 @@ class GearmanClientThread(object):
                     filter(Device.id == data).first()
                 vip = session.query(Vip).\
                     filter(Vip.device == data).first()
-                submit_vip_job(
-                    'REMOVE', dev.name, str(ipaddress.IPv4Address(vip.ip))
-                )
-                vip.device = None
+                if vip:
+                    submit_vip_job(
+                        'REMOVE', dev.name, str(ipaddress.IPv4Address(vip.ip))
+                    )
                 job_data = {"hpcs_action": "DELETE"}
 
             status, response = self._send_message(job_data, 'hpcs_response')
@@ -180,7 +281,7 @@ class GearmanClientThread(object):
                 filter(LoadBalancer.id == self.lbid).\
                 first()
             if not status:
-                self.logger.error(
+                LOG.error(
                     "Failed Gearman delete for LB {0}".format(lb.id)
                 )
                 self._set_error(data, response, session)
@@ -211,6 +312,10 @@ class GearmanClientThread(object):
         device = session.query(Device).\
             filter(Device.id == device_id).\
             first()
+        if device is None:
+            # Device already deleted, probably a race between the OFFLINE check
+            # and auto-failover
+            return
         device.status = 'ERROR'
         for lb in lbs:
             lb.status = 'ERROR'
@@ -259,6 +364,15 @@ class GearmanClientThread(object):
             }
 
             degraded = []
+            if lbs is None:
+                LOG.error(
+                    'Attempting to send empty LB data for device {0} ({1}), '
+                    'something went wrong'.format(data, self.host)
+                )
+                self._set_error(data, "LB config error", session)
+                session.commit()
+                return
+
             for lb in lbs:
                 lb_data = {
                     'name': lb.name,
@@ -320,11 +434,31 @@ class GearmanClientThread(object):
                     if lb.id in degraded:
                         lb.status = 'DEGRADED'
                         lb.errmsg = "A node on the load balancer has failed"
+                    elif lb.status == 'ERROR':
+                        # Do nothing because something else failed in the mean
+                        # time
+                        pass
+                    elif lb.status == 'BUILD':
+                        # Do nothing, stay in BUILD state until floating IP
+                        # assign finishes
+                        pass
                     else:
                         lb.status = 'ACTIVE'
                         lb.errmsg = None
-
+            device = session.query(Device).\
+                filter(Device.id == data).\
+                first()
+            if device is None:
+                # Shouldn't hit here, but just to be safe
+                session.commit()
+                return
+            device_name = device.name
+            device_status = device.status
             session.commit()
+            if device_status == 'BUILD':
+                submit_vip_job(
+                    'ASSIGN', device_name, None
+                )
 
     def _send_message(self, message, response_name):
         job_status = self.gearman_client.submit_job(
@@ -333,15 +467,15 @@ class GearmanClientThread(object):
         )
         if job_status.state == 'UNKNOWN':
             # Gearman server connection failed
-            self.logger.error('Could not talk to gearman server')
+            LOG.error('Could not talk to gearman server')
             return False, "System error communicating with load balancer"
         if job_status.timed_out:
             # Job timed out
-            self.logger.warning(
+            LOG.warning(
                 'Gearman timeout talking to {0}'.format(self.host)
             )
             return False, "Timeout error communicating with load balancer"
-        self.logger.debug(job_status.result)
+        LOG.debug(job_status.result)
         if 'badRequest' in job_status.result:
             error = job_status.result['badRequest']['validationErrors']
             return False, error['message']
@@ -351,9 +485,9 @@ class GearmanClientThread(object):
                 error = job_status.result['hpcs_error']
             else:
                 error = 'Load Balancer error'
-            self.logger.error(
+            LOG.error(
                 'Gearman error response from {0}: {1}'.format(self.host, error)
             )
             return False, error
-        self.logger.info('Gearman success from {0}'.format(self.host))
+        LOG.info('Gearman success from {0}'.format(self.host))
         return True, job_status.result
