@@ -12,15 +12,21 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from sqlalchemy import Table, Column, Integer, ForeignKey, create_engine
-from sqlalchemy import INTEGER, VARCHAR, BIGINT
-from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.orm import relationship, backref, sessionmaker, Session
+
+import ConfigParser
 import sqlalchemy.types as types
 import time
-import ConfigParser
+
+from oslo.config import cfg
 from pecan import conf
-import logging
+from sqlalchemy import Table, Column, Integer, ForeignKey, create_engine
+from sqlalchemy import INTEGER, VARCHAR, BIGINT, DATETIME
+from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.orm import relationship, backref, sessionmaker, Session
+
+from libra.openstack.common import log
+
+LOG = log.getLogger(__name__)
 
 
 DeclarativeBase = declarative_base()
@@ -145,6 +151,28 @@ class HealthMonitor(DeclarativeBase):
     path = Column(u'path', VARCHAR(length=2000))
 
 
+class Billing(DeclarativeBase):
+    __tablename__ = 'billing'
+    id = Column(u'id', Integer, primary_key=True, nullable=False)
+    name = Column(u'name', VARCHAR(length=128), nullable=False)
+    last_update = Column(u'last_update', DATETIME(), nullable=False)
+
+
+class Stats(DeclarativeBase):
+    """stats model"""
+    __tablename__ = 'stats'
+    #column definitions
+    id = Column(u'id', BIGINT(), primary_key=True, nullable=False)
+    lbid = Column(
+        u'lbid', BIGINT(), ForeignKey('loadbalancers.id'), primary_key=True,
+        nullable=False
+    )
+    period_start = Column(u'period_start', DATETIME(), nullable=False)
+    period_end = Column(u'period_end', DATETIME(), nullable=False)
+    bytes_out = Column(u'bytes_out', BIGINT(), nullable=False)
+    status = Column(u'status', VARCHAR(length=50), nullable=False)
+
+
 class RoutingSession(Session):
     """ Try to use the first engine provided.  If this fails use the next in
         sequence and so on.  Reset to the first after 60 seconds
@@ -170,8 +198,11 @@ class RoutingSession(Session):
         return engine
 
     def _build_engines(self):
+        # We have to use ConfigParser here because with oslo.config, we need
+        # to know the section names before parsing.
         config = ConfigParser.SafeConfigParser()
-        config.read([conf.conffile])
+        config.read(cfg.CONF['config_file'])
+
         if 'debug' in conf.app and conf.app.debug:
             echo = True
         else:
@@ -180,11 +211,11 @@ class RoutingSession(Session):
         for section in conf.database:
             db_conf = config._sections[section]
 
-            conn_string = '''mysql+mysqlconnector://%s:%s@%s:%d/%s''' % (
+            conn_string = '''mysql+mysqlconnector://%s:%s@%s:%s/%s''' % (
                 db_conf['username'],
                 db_conf['password'],
                 db_conf['host'],
-                db_conf.get('port', 3306),
+                db_conf['port'],
                 db_conf['schema']
             )
 
@@ -213,7 +244,6 @@ class RoutingSession(Session):
 class db_session(object):
     def __init__(self):
         self.session = None
-        self.logger = logging.getLogger(__name__)
 
     def __enter__(self):
         for x in xrange(10):
@@ -222,7 +252,7 @@ class db_session(object):
                 self.session.execute("SELECT 1")
                 return self.session
             except:
-                self.logger.error(
+                LOG.error(
                     'Could not connect to DB server: {0}'.format(
                         RoutingSession.engines[RoutingSession.use_engine].url
                     )
@@ -231,7 +261,7 @@ class db_session(object):
                 RoutingSession.use_engine += 1
                 if RoutingSession.use_engine == RoutingSession.engines_count:
                     RoutingSession.use_engine = 0
-        self.logger.error('Could not connect to any DB server')
+        LOG.error('Could not connect to any DB server')
         return None
 
     def __exit__(self, type, value, traceback):
