@@ -12,11 +12,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import socket
+
+from oslo.config import cfg
+
 from libra import __version__ as libra_version
 from libra import __release__ as libra_release
 from libra.common.exc import DeletedStateError
 from libra.common.faults import BadRequest
-from libra.worker.drivers.base import LoadBalancerDriver
+from libra.openstack.common import log
+from libra.worker.drivers import base
+
+LOG = log.getLogger(__name__)
 
 
 class LBaaSController(object):
@@ -34,8 +41,7 @@ class LBaaSController(object):
     OBJ_STORE_ENDPOINT_FIELD = 'hpcs_object_store_endpoint'
     OBJ_STORE_TOKEN_FIELD = 'hpcs_object_store_token'
 
-    def __init__(self, logger, driver, json_msg):
-        self.logger = logger
+    def __init__(self, driver, json_msg):
         self.driver = driver
         self.msg = json_msg
 
@@ -45,7 +51,7 @@ class LBaaSController(object):
         """
 
         if self.ACTION_FIELD not in self.msg:
-            self.logger.error("Missing `%s` value" % self.ACTION_FIELD)
+            LOG.error("Missing `%s` value" % self.ACTION_FIELD)
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             return self.msg
 
@@ -66,17 +72,67 @@ class LBaaSController(object):
                 return self._action_archive()
             elif action == 'STATS':
                 return self._action_stats()
+            elif action == 'PING':
+                return self._action_ping()
+            elif action == 'DIAGNOSTICS':
+                return self._action_diagnostic()
             else:
-                self.logger.error("Invalid `%s` value: %s" %
-                                  (self.ACTION_FIELD, action))
+                LOG.error("Invalid `%s` value: %s", self.ACTION_FIELD, action)
                 self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                 return self.msg
         except Exception as e:
-            self.logger.error("Controller exception: %s, %s" %
-                              (e.__class__, e))
+            LOG.error("Controller exception: %s, %s", e.__class__, e)
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             return self.msg
 
+    def _action_diagnostic(self):
+        """
+        Returns the results of a diagnostic run
+
+        This message is used to see if the worker that was built will actually
+        function as a load balancer
+        """
+        # Gearman test
+        self.msg['gearman'] = []
+        for host_port in cfg.CONF['gearman']['servers']:
+            host, port = host_port.split(':')
+            try:
+                self._check_host(host, int(port))
+            except:
+                self.msg['gearman'].append(
+                    {'host': host, 'status': self.RESPONSE_FAILURE}
+                )
+            else:
+                self.msg['gearman'].append(
+                    {'host': host, 'status': self.RESPONSE_SUCCESS}
+                )
+        # Outgoing network test
+        try:
+            # TODO: make this configurable
+            self._check_host('google.com', 80)
+        except:
+            self.msg['network'] = self.RESPONSE_FAILURE
+        else:
+            self.msg['network'] = self.RESPONSE_SUCCESS
+
+        self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
+        return self.msg
+
+    def _check_host(self, ip, port):
+        # TCP connect check to see if floating IP was assigned correctly
+        sock = socket.socket()
+        sock.settimeout(5)
+        try:
+            sock.connect((ip, port))
+            sock.close()
+            return True
+        except socket.error:
+            LOG.error(
+                "TCP connect error to gearman server {0}"
+                .format(ip)
+            )
+            raise
+
     def _action_discover(self):
         """
         Return service discovery information.
@@ -104,7 +160,7 @@ class LBaaSController(object):
         except NotImplementedError:
             pass
         except Exception as e:
-            self.logger.error("Selected driver failed initialization.")
+            LOG.error("Selected driver failed initialization.")
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             return self.msg
 
@@ -130,13 +186,13 @@ class LBaaSController(object):
                 try:
                     self.driver.add_protocol(current_lb['protocol'], port)
                 except NotImplementedError:
-                    self.logger.error(
+                    LOG.error(
                         "Selected driver does not support setting protocol."
                     )
                     self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                     return self.msg
                 except Exception as e:
-                    self.logger.error(
+                    LOG.error(
                         "Failure trying to set protocol: %s, %s" %
                         (e.__class__, e)
                     )
@@ -146,26 +202,26 @@ class LBaaSController(object):
             if 'algorithm' in current_lb:
                 algo = current_lb['algorithm'].upper()
                 if algo == 'ROUND_ROBIN':
-                    algo = LoadBalancerDriver.ROUNDROBIN
+                    algo = base.LoadBalancerDriver.ROUNDROBIN
                 elif algo == 'LEAST_CONNECTIONS':
-                    algo = LoadBalancerDriver.LEASTCONN
+                    algo = base.LoadBalancerDriver.LEASTCONN
                 else:
-                    self.logger.error("Invalid algorithm: %s" % algo)
+                    LOG.error("Invalid algorithm: %s" % algo)
                     self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                     return self.msg
             else:
-                algo = LoadBalancerDriver.ROUNDROBIN
+                algo = base.LoadBalancerDriver.ROUNDROBIN
 
             try:
                 self.driver.set_algorithm(current_lb['protocol'], algo)
             except NotImplementedError:
-                self.logger.error(
+                LOG.error(
                     "Selected driver does not support setting algorithm."
                 )
                 self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                 return self.msg
             except Exception as e:
-                self.logger.error(
+                LOG.error(
                     "Selected driver failed setting algorithm."
                 )
                 self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
@@ -188,13 +244,13 @@ class LBaaSController(object):
                                             monitor['attempts'],
                                             monitor['path'])
                 except NotImplementedError:
-                    self.logger.error(
+                    LOG.error(
                         "Selected driver does not support adding healthchecks."
                     )
                     self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                     return self.msg
                 except Exception as e:
-                    self.logger.error(
+                    LOG.error(
                         "Selected driver failed adding healthchecks: %s, %s" %
                         (e.__class__, e)
                     )
@@ -239,25 +295,25 @@ class LBaaSController(object):
                 except NotImplementedError:
                     lb_node['condition'] = self.NODE_ERR
                     error = "Selected driver does not support adding a server"
-                    self.logger.error(error)
+                    LOG.error(error)
                     self.msg[self.ERROR_FIELD] = error
                     self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                     return self.msg
                 except Exception as e:
                     lb_node['condition'] = self.NODE_ERR
                     error = "Failure adding server %s: %s" % (node_id, e)
-                    self.logger.error(error)
+                    LOG.error(error)
                     self.msg[self.ERROR_FIELD] = error
                     self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
                     return self.msg
                 else:
-                    self.logger.debug("Added server: %s:%s" % (address, port))
+                    LOG.debug("Added server: %s:%s" % (address, port))
                     lb_node['condition'] = self.NODE_OK
 
         try:
             self.driver.create()
         except NotImplementedError:
-            self.logger.error(
+            LOG.error(
                 "Selected driver does not support CREATE action."
             )
             for current_lb in lb_list:
@@ -265,12 +321,12 @@ class LBaaSController(object):
                     lb_node['condition'] = self.NODE_ERR
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         except Exception as e:
-            self.logger.error("CREATE failed: %s, %s" % (e.__class__, e))
+            LOG.error("CREATE failed: %s, %s" % (e.__class__, e))
             for lb_node in current_lb['nodes']:
                 lb_node['condition'] = self.NODE_ERR
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         else:
-            self.logger.info("Activated load balancer changes")
+            LOG.info("Activated load balancer changes")
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
 
         return self.msg
@@ -280,12 +336,12 @@ class LBaaSController(object):
         try:
             self.driver.suspend()
         except NotImplementedError:
-            self.logger.error(
+            LOG.error(
                 "Selected driver does not support SUSPEND action."
             )
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         except Exception as e:
-            self.logger.error("SUSPEND failed: %s, %s" % (e.__class__, e))
+            LOG.error("SUSPEND failed: %s, %s" % (e.__class__, e))
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         else:
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
@@ -296,12 +352,12 @@ class LBaaSController(object):
         try:
             self.driver.enable()
         except NotImplementedError:
-            self.logger.error(
+            LOG.error(
                 "Selected driver does not support ENABLE action."
             )
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         except Exception as e:
-            self.logger.error("ENABLE failed: %s, %s" % (e.__class__, e))
+            LOG.error("ENABLE failed: %s, %s" % (e.__class__, e))
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         else:
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
@@ -312,12 +368,12 @@ class LBaaSController(object):
         try:
             self.driver.delete()
         except NotImplementedError:
-            self.logger.error(
+            LOG.error(
                 "Selected driver does not support DELETE action."
             )
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         except Exception as e:
-            self.logger.error("DELETE failed: %s, %s" % (e.__class__, e))
+            LOG.error("DELETE failed: %s, %s" % (e.__class__, e))
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
         else:
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
@@ -373,48 +429,90 @@ class LBaaSController(object):
             self.driver.archive(method, params)
         except NotImplementedError:
             error = "Selected driver does not support ARCHIVE action."
-            self.logger.error(error)
+            LOG.error(error)
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = error
         except Exception as e:
-            self.logger.error("ARCHIVE failed: %s, %s" % (e.__class__, e))
+            LOG.error("ARCHIVE failed: %s, %s" % (e.__class__, e))
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = str(e)
         else:
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
         return self.msg
 
-    def _action_stats(self):
+    def _action_ping(self):
         """
-        Get load balancer statistics.
+        Get load balancer and node status.
 
         We push responsibility for knowing what state a load balancer
-        current is in to the driver. Trying to get statistics for a LB that
+        current is in to the driver. Trying to get status for a LB that
         has been deleted is an error.
         """
 
         try:
-            # TODO: Do something with the returned statistics
-            stats = self.driver.get_stats()
+            nodes = self.driver.get_status()
         except NotImplementedError:
-            error = "Selected driver does not support STATS action."
-            self.logger.error(error)
+            error = "Selected driver does not support PING action."
+            LOG.error(error)
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = error
         except DeletedStateError:
-            self.logger.info("Invalid operation STATS on a deleted LB")
+            error = "Invalid operation PING on a deleted LB."
+            LOG.error(error)
             self.msg['status'] = 'DELETED'
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
+            self.msg[self.ERROR_FIELD] = error
         except Exception as e:
-            self.logger.error("STATS failed: %s, %s" % (e.__class__, e))
+            LOG.error("PING failed: %s, %s" % (e.__class__, e))
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             self.msg[self.ERROR_FIELD] = str(e)
         else:
-            node_status = stats.node_status_map()
             self.msg['nodes'] = []
-            for node in node_status.keys():
-                self.msg['nodes'].append({'id': node,
-                                          'status': node_status[node]})
+            for node, status in nodes:
+                self.msg['nodes'].append({'id': node, 'status': status})
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
 
         return self.msg
+
+    def _action_stats(self):
+        """
+        Get load balancer statistics
+
+        This type of request gets the number of bytes out for each load
+        balancer defined on the device. If both a TCP and HTTP load
+        balancer exist, we report on each in a single response.
+        """
+
+        try:
+            start, end, statistics = self.driver.get_statistics()
+        except NotImplementedError:
+            error = "Selected driver does not support STATS action."
+            LOG.error(error)
+            self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
+            self.msg[self.ERROR_FIELD] = error
+            return self.msg
+        except DeletedStateError:
+            error = "Invalid operation STATS on a deleted LB."
+            LOG.error(error)
+            self.msg['status'] = 'DELETED'
+            self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
+            self.msg[self.ERROR_FIELD] = error
+            return self.msg
+        except Exception as e:
+            LOG.error("STATS failed: %s, %s" % (e.__class__, e))
+            self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
+            self.msg[self.ERROR_FIELD] = str(e)
+            return self.msg
+
+        self.msg['utc_start'] = start
+        self.msg['utc_end'] = end
+        self.msg['loadBalancers'] = []
+
+        # We should have a list of tuples pairing the number of bytes
+        # out with the protocol/LB.
+        for proto, bytes_out in statistics:
+            self.msg['loadBalancers'].append({'protocol': proto,
+                                              'bytes_out': bytes_out})
+
+        self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
+        return self.msg
