@@ -26,8 +26,8 @@ from libra.worker.drivers.haproxy.services_base import ServicesBase
 
 class HAProxyDriver(LoadBalancerDriver):
 
-    def __init__(self, ossvc, user, group):
-        self.haproxy_log = '/mnt/log/haproxy.log'
+    def __init__(self, ossvc, user, group, haproxy_logfile=None):
+        self.haproxy_log = haproxy_logfile
         self.user = user
         self.group = group
         ossvc_driver = importutils.import_class(ossvc)
@@ -377,8 +377,7 @@ class HAProxyDriver(LoadBalancerDriver):
 
     def create(self):
         self.ossvc.write_config(self._config_to_string())
-        self.ossvc.service_stop()
-        self.ossvc.service_start()
+        self.ossvc.service_reload()
 
     def suspend(self):
         self.ossvc.service_stop()
@@ -394,8 +393,11 @@ class HAProxyDriver(LoadBalancerDriver):
         # restart, otherwise the log file will be kept open and not reappear.
         self.ossvc.syslog_restart()
 
-    def get_stats(self, protocol=None):
-        return self.ossvc.get_stats(protocol)
+    def get_status(self, protocol=None):
+        return self.ossvc.get_status(protocol)
+
+    def get_statistics(self):
+        return self.ossvc.get_statistics()
 
     def archive(self, method, params):
         """
